@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Init;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.3';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -58,7 +58,7 @@ sub logLevelInit {
     $logger = $class->localConfig->{userLogger} || $logger;
     eval "require $logger";
     die $@ if ($@);
-    $class->userLogger( $logger->new( $class->localConfig ), user => 1 );
+    $class->userLogger( $logger->new( $class->localConfig, user => 1 ) );
     $class->logger->debug("User logger $logger loaded");
 }
 
@@ -66,7 +66,9 @@ sub logLevelInit {
 # adapt server signature
 sub serverSignatureInit {
     my $class = shift;
-    $class->setServerSignature("Lemonldap::NG/$VERSION");
+    require Lemonldap::NG::Handler;
+    my $version = $Lemonldap::NG::Handler::VERSION;
+    $class->setServerSignature("Lemonldap::NG/$version");
 }
 
 ## @ifn protected void statusInit()
