@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Reload;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -33,9 +33,10 @@ sub onReload {
 # @return true if config is up to date or if reload config succeeded
 sub checkConf {
     my ( $class, $force ) = @_;
+    $class->logger->debug("Check configuration for $class");
     my $prm = { local => !$force, localPrm => $class->localConfig };
-    $prm->{noCache} = 1 unless ( $class->cfgNum );
     my $conf = $class->confAcc->getConf($prm);
+    chomp $Lemonldap::NG::Common::Conf::msg;
 
     unless ( ref($conf) ) {
         $class->logger->error(
@@ -44,29 +45,41 @@ sub checkConf {
         return $force ? 0 : $class->cfgNum ? 1 : 0;
     }
 
-    if ( !$class->cfgNum or $class->cfgNum != $conf->{cfgNum} ) {
-        $class->logger->debug(
-"Get configuration $conf->{cfgNum} ($Lemonldap::NG::Common::Conf::msg)"
-        );
-        $class->lastCheck( time() );
+    if ($Lemonldap::NG::Common::Conf::msg) {
+        if ( $Lemonldap::NG::Common::Conf::msg =~ /Error:/ ) {
+            $class->logger->error($Lemonldap::NG::Common::Conf::msg);
+        }
+        elsif ( $Lemonldap::NG::Common::Conf::msg =~ /Warn:/ ) {
+            $class->logger->warn($Lemonldap::NG::Common::Conf::msg);
+        }
+        else {
+            $class->logger->debug($Lemonldap::NG::Common::Conf::msg);
+        }
+    }
+    if ( $force or !$class->cfgNum or $class->cfgNum != $conf->{cfgNum} ) {
+        $class->logger->debug("Get configuration $conf->{cfgNum}");
         unless ( $class->cfgNum( $conf->{cfgNum} ) ) {
             $class->logger->error('No configuration available');
             return 0;
         }
         $class->configReload($conf);
-    }
-    $class->logger->debug("$class: configuration is up to date");
-    foreach (@_onReload) {
-        my ( $obj, $sub ) = @$_;
-        if ($obj) {
-            $class->logger->debug( 'Launching ' . ref($obj) . "->$sub(conf)" );
-            unless ( $obj->$sub($conf) ) {
-                $class->logger->error( "Underlying object can't load conf ("
-                      . ref($obj)
-                      . "->$sub)" );
+        foreach (@_onReload) {
+            my ( $obj, $sub ) = @$_;
+            if ($obj) {
+                $class->logger->debug(
+                    'Launching ' . ref($obj) . "->$sub(conf)" );
+                unless ( $obj->$sub($conf) ) {
+                    $class->logger->error( "Underlying object can't load conf ("
+                          . ref($obj)
+                          . "->$sub)" );
+                    return 0;
+                }
             }
         }
     }
+    $class->tsv->{checkTime} = $conf->{checkTime} if ( $conf->{checkTime} );
+    $class->lastCheck( time() );
+    $class->logger->debug("$class: configuration is up to date");
     return 1;
 }
 
@@ -165,13 +178,12 @@ sub configReload {
 sub jailInit {
     my ( $class, $conf ) = @_;
 
-    $class->tsv->{jail} = Lemonldap::NG::Handler::Main::Jail->new(
-        {
-            'useSafeJail'     => $conf->{useSafeJail},
-            'customFunctions' => $conf->{customFunctions}
+    $class->tsv->{jail} = Lemonldap::NG::Handler::Main::Jail->new( {
+            useSafeJail     => $conf->{useSafeJail},
+            customFunctions => $conf->{customFunctions},
         }
     );
-    $class->tsv->{jail}->build_jail($class);
+    $class->tsv->{jail}->build_jail( $class, $conf->{require} );
 }
 
 ## @imethod protected void defaultValuesInit(hashRef args)
@@ -180,19 +192,17 @@ sub jailInit {
 sub defaultValuesInit {
     my ( $class, $conf ) = @_;
 
-    $class->tsv->{$_} = $conf->{$_} foreach (
-        qw(
+    $class->tsv->{$_} = $conf->{$_} foreach ( qw(
         cookieExpiration        cookieName         customFunctions httpOnly
         securedCookie           timeout            timeoutActivity
         timeoutActivityInterval useRedirectOnError useRedirectOnForbidden
-        useSafeJail             whatToTrace
+        useSafeJail             whatToTrace        handlerInternalCache
         )
     );
 
     $class->tsv->{cipher} = Lemonldap::NG::Common::Crypto->new( $conf->{key} );
 
     foreach my $opt (qw(https port maintenance)) {
-        next unless defined $conf->{$opt};
 
         # Record default value in key '_'
         $class->tsv->{$opt} = { _ => $conf->{$opt} };
@@ -201,9 +211,10 @@ sub defaultValuesInit {
         if ( $conf->{vhostOptions} ) {
             my $name = 'vhost' . ucfirst($opt);
             foreach my $vhost ( keys %{ $conf->{vhostOptions} } ) {
+                $conf->{vhostOptions}->{$vhost} ||= {};
                 my $val = $conf->{vhostOptions}->{$vhost}->{$name};
 
-                # Keep default value if $val is negative
+                # Keep global value if $val is negative
                 if ( defined $val and $val >= 0 ) {
                     $class->logger->debug(
                         "Options $opt for vhost $vhost: $val");
@@ -216,6 +227,8 @@ sub defaultValuesInit {
         foreach my $vhost ( keys %{ $conf->{vhostOptions} } ) {
             $class->tsv->{type}->{$vhost} =
               $conf->{vhostOptions}->{$vhost}->{vhostType};
+            $class->tsv->{authnLevel}->{$vhost} =
+              $conf->{vhostOptions}->{$vhost}->{vhostAuthnLevel};
         }
     }
     return 1;
@@ -324,7 +337,7 @@ sub sessionStorageInit {
                   . Data::Dumper->new( [ $class->tsv->{sessionCacheOptions} ] )
                   ->Terse(1)->Indent(0)->Dump;    # To send params on one line
             }
-            print { $class->tsv->{statusPipe} } "RELOADCACHE $params\n";
+            $class->tsv->{statusPipe}->print("RELOADCACHE $params\n");
         }
     }
     return 1;
@@ -339,12 +352,17 @@ sub headersInit {
 
     # Creation of the subroutine which will generate headers
     foreach my $vhost ( keys %{$headers} ) {
+        unless ($vhost) {
+            $class->logger->warn('Empty vhost in headers, skipping');
+            next;
+        }
+        $headers->{$vhost} ||= {};
         my %headers = %{ $headers->{$vhost} };
         $class->tsv->{headerList}->{$vhost} = [ keys %headers ];
         my $sub = '';
         foreach ( keys %headers ) {
             my $val = $class->substitute( $headers{$_} );
-            $sub .= "('$_' => $val),";
+            $sub .= "('$_' => $val || ''),";
         }
 
         unless ( $class->tsv->{forgeHeaders}->{$vhost} =
@@ -368,7 +386,7 @@ sub postUrlInit {
     foreach my $vhost ( keys %{ $conf->{post} } ) {
 
         #  Browse all POST URI
-        foreach my $url ( keys %{ $conf->{post}->{$vhost} } ) {
+        foreach my $url ( keys %{ $conf->{post}->{$vhost} || {} } ) {
             my $d = $conf->{post}->{$vhost}->{$url};
             $class->logger->debug("Compiling POST data for $url");
 
@@ -386,8 +404,7 @@ sub postUrlInit {
                 $class->buildSub($sub) )
             {
                 $class->tsv->{maintenance}->{$vhost} = 1;
-                $class->logger->error(
-                    "$class: Unable to build post datas: "
+                $class->logger->error( "$class: Unable to build post data: "
                       . $class->tsv->{jail}->error );
             }
 
@@ -427,15 +444,14 @@ sub conditionSub {
             $url
             ? (
                 sub {
-                    $_[0]->{_logout} = $url;
+                    $_[1]->{_logout} = $url;
                     return 0;
                 },
                 0
               )
             : (
                 sub {
-                    $_[0]->{_logout} =
-                      &{ $class->tsv->{portal} }();
+                    $_[1]->{_logout} = $class->tsv->{portal}->();
                     return 0;
                 },
                 0
@@ -456,41 +472,39 @@ sub conditionSub {
 
     # logout_app
     if ( $cond =~ /^logout_app(?:\s+(.*))?$/i ) {
-        my $u = $1 || &{ $class->tsv->{portal} }();
+        my $u = $1 || $class->tsv->{portal}->();
         eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
         return (
             sub {
-                $Lemonldap::NG::Handler::ApacheMP2::Main::request
-                  ->add_output_filter(
+                $_[0]->{env}->{'psgi.r'}->add_output_filter(
                     sub {
                         return $class->redirectFilter( $u, @_ );
                     }
-                  );
+                );
                 1;
             },
             0
         );
     }
     elsif ( $cond =~ /^logout_app_sso(?:\s+(.*))?$/i ) {
-        my $u = $1 || &{ $class->tsv->{portal} }();
+        my $u = $1 || $class->tsv->{portal}->();
         eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
         return (
             sub {
 
-                #TODO: check this
+                my ($req) = @_;
                 $class->localUnlog;
-                $Lemonldap::NG::Handler::ApacheMP2::Main::request
-                  ->add_output_filter(
+                $req->{env}->{'psgi.r'}->add_output_filter(
                     sub {
                         my $r = $_[0]->r;
                         return $class->redirectFilter(
                             &{ $class->tsv->{portal} }() . "?url="
-                              . $class->encodeUrl($u)
+                              . $class->encodeUrl( $req, $u )
                               . "&logout=1",
                             @_
                         );
                     }
-                  );
+                );
                 1;
             },
             0
@@ -533,11 +547,13 @@ sub substitute {
 
     # substitute special vars, just for retro-compatibility
     $expr =~ s/\$date\b/&date/sg;
-    $expr =~ s/\$vhost\b/&hostname/sg;
-    $expr =~ s/\$ip\b/&remote_ip/sg;
+    $expr =~ s/\$vhost\b/\$ENV{HTTP_HOST}/sg;
+    $expr =~ s/\$ip\b/\$ENV{REMOTE_ADDR}/sg;
 
-    # substitute vars with session datas, excepts special vars $_ and $\d+
-    $expr =~ s/\$(?!ENV)(_*[a-zA-Z]\w*)/\$session->{$1}/sg;
+    # substitute vars with session data, excepts special vars $_ and $\d+
+    $expr =~ s/\$(?!(?:ENV|env)\b)([_a-zA-Z]\w*)/\$s->{$1}/sg;
+    $expr =~ s/\$ENV\{/\$r->{env}->\{/g;
+    $expr =~ s/\$env->\{/\$r->{env}->\{/g;
 
     return $expr;
 }
@@ -545,7 +561,7 @@ sub substitute {
 sub buildSub {
     my ( $class, $val ) = @_;
     my $res =
-      $class->tsv->{jail}->jail_reval("sub{my \$session=\$_[0];return($val)}");
+      $class->tsv->{jail}->jail_reval("sub{my (\$r,\$s)=\@_;return($val)}");
     unless ($res) {
         $class->logger->error( $class->tsv->{jail}->error );
     }
