@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.3';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -341,8 +341,12 @@ sub hideCookie {
     my ( $class, $req ) = @_;
     $class->logger->debug("removing cookie");
     my $cookie = $req->env->{HTTP_COOKIE};
-    my $cn     = $class->tsv->{cookieName};
-    $cookie =~ s/$cn(http)?=[^,;]*[,;\s]*//og;
+    $class->logger->debug("Cookies -> $cookie");
+    my $cn = $class->tsv->{cookieName};
+    $class->logger->debug("CookieName -> $cn");
+    $cookie =~ s/\b$cn(http)?=[^,;]*[,;\s]*//og;
+    $class->logger->debug("newCookies -> $cookie");
+
     if ($cookie) {
         $class->set_header_in( $req, 'Cookie' => $cookie );
     }
@@ -398,12 +402,8 @@ sub fetchId {
     my ( $class, $req ) = @_;
     my $t                 = $req->{env}->{HTTP_COOKIE} or return 0;
     my $vhost             = $class->resolveAlias($req);
-    my $lookForHttpCookie = (
-        $class->tsv->{securedCookie} =~ /^(2|3)$/
-          and !( defined( $class->tsv->{https}->{$vhost} ) )
-        ? $class->tsv->{https}->{$vhost}
-        : $class->tsv->{https}->{_}
-    );
+    my $lookForHttpCookie = ( $class->tsv->{securedCookie} =~ /^(2|3)$/
+          and not $class->_isHttps( $req, $vhost ) );
     my $cn = $class->tsv->{cookieName};
     my $value =
       $lookForHttpCookie
@@ -437,8 +437,8 @@ sub retrieveSession {
     #     (15 seconds)
     if (    defined $class->data->{_session_id}
         and $id eq $class->data->{_session_id}
-        and ( $now - $class->dataUpdate < $class->tsv->{handlerInternalCache} )
-      )
+        and
+        ( $now - $class->dataUpdate < $class->tsv->{handlerInternalCache} ) )
     {
         $class->logger->debug("Get session $id from Handler internal cache");
         return $class->data;
@@ -446,8 +446,7 @@ sub retrieveSession {
 
     # 2. Get the session from cache or backend
     my $session = $req->data->{session} = (
-        Lemonldap::NG::Common::Session->new(
-            {
+        Lemonldap::NG::Common::Session->new( {
                 storageModule        => $class->tsv->{sessionStorageModule},
                 storageModuleOptions => $class->tsv->{sessionStorageOptions},
                 cacheModule          => $class->tsv->{sessionCacheModule},
@@ -533,23 +532,67 @@ sub retrieveSession {
     }
 }
 
+## @cmethod private int _getPort(string s)
+# Returns the port on which this vhost is accessed
+# @param $s VHost name
+# @return PORT
+
+sub _getPort {
+
+    my ( $class, $req, $vhost ) = @_;
+
+    if ( defined $class->tsv->{port}->{$vhost}
+        and ( $class->tsv->{port}->{$vhost} > 0 ) )
+    {
+        return $class->tsv->{port}->{$vhost};
+    }
+    else {
+        if ( defined $class->tsv->{port}->{_}
+            and ( $class->tsv->{port}->{_} > 0 ) )
+        {
+            return $class->tsv->{port}->{_};
+        }
+        else {
+            return $req->{env}->{SERVER_PORT};
+        }
+    }
+}
+## @cmethod private boot _isHttps(string s)
+# Returns whether this VHost should he accessed
+# via HTTPS
+# @param $s VHost name
+# @return RUE if the vhost should be accessed over HTTPS
+sub _isHttps {
+
+    my ( $class, $req, $vhost ) = @_;
+
+    if ( defined $class->tsv->{https}->{$vhost}
+        and ( $class->tsv->{https}->{$vhost} > -1 ) )
+    {
+        return $class->tsv->{https}->{$vhost};
+    }
+    else {
+        if ( defined $class->tsv->{https}->{_}
+            and ( $class->tsv->{https}->{_} > -1 ) )
+        {
+            return $class->tsv->{https}->{_};
+        }
+        else {
+            return ( ( uc( $req->{env}->{HTTPS} ) || "OFF" ) eq "ON" );
+        }
+    }
+}
+
 ## @cmethod private string _buildUrl(string s)
 # Transform /<s> into http(s?)://<host>:<port>/s
 # @param $s path
 # @return URL
 sub _buildUrl {
     my ( $class, $req, $s ) = @_;
-    my $realvhost = $req->{env}->{HTTP_HOST};
-    my $vhost     = $class->resolveAlias($req);
-    my $_https    = (
-        defined( $class->tsv->{https}->{$vhost} )
-        ? $class->tsv->{https}->{$vhost}
-        : $class->tsv->{https}->{_}
-    );
-    my $portString =
-         $class->tsv->{port}->{$vhost}
-      || $class->tsv->{port}->{_}
-      || $req->{env}->{SERVER_PORT};
+    my $realvhost  = $req->{env}->{HTTP_HOST};
+    my $vhost      = $class->resolveAlias($req);
+    my $_https     = $class->_isHttps( $req, $vhost );
+    my $portString = $class->_getPort( $req, $vhost );
     $portString = (
              ( $realvhost =~ /:\d+/ )
           or ( $_https  && $portString == 443 )
@@ -582,7 +625,7 @@ sub isUnprotected {
 }
 
 ## @rmethod void sendHeaders()
-# Launch function compiled by forgeHeadersInit() for the current virtual host
+# Launch function compiled by headersInit() for the current virtual host
 sub sendHeaders {
     my ( $class, $req, $session ) = @_;
     my $vhost = $class->resolveAlias($req);
@@ -603,6 +646,27 @@ sub sendHeaders {
     }
 }
 
+## @rfunction array ref checkHeaders()
+# Return computed headers by headersInit() for the current virtual host
+# [ { key => 'header1', value => 'value1' }, { key => 'header2', value => 'value2' }, ...]
+sub checkHeaders {
+    my ( $class, $req, $session ) = @_;
+    my $vhost         = $class->resolveAlias($req);
+    my $array_headers = [];
+    if ( defined $class->tsv->{forgeHeaders}->{$vhost} ) {
+
+        # Create array of hashes with headers
+        my %headers =
+          $class->tsv->{forgeHeaders}->{$vhost}->( $req, $session );
+        foreach my $h ( sort keys %headers ) {
+            defined $headers{$h}
+              ? push @$array_headers, { key => $h, value => $headers{$h} }
+              : push @$array_headers, { key => $h, value => '' };
+        }
+    }
+    return $array_headers;
+}
+
 ## @rmethod void cleanHeaders()
 # Unset HTTP headers, when sendHeaders is skipped
 sub cleanHeaders {
@@ -716,7 +780,11 @@ sub postInputFilter {
         my %data =
           $class->tsv->{inputPostData}->{$vhost}->{$uri}->( $req, $session );
         foreach ( keys %data ) {
-            $data{$_} = uri_escape( $data{$_} );
+            my $post_key   = uri_escape($_);
+            my $post_value = uri_escape( $data{$_} );
+            delete $data{$_};
+            $data{$post_key} = $post_value;
+            $class->logger->debug("Send key $post_key with value $post_value");
         }
         $class->setPostParams( $req, \%data );
     }
@@ -745,7 +813,8 @@ sub postJavascript {
       : "form.submit();\n";
 
     my $jqueryUrl = $formParams->{jqueryUrl} || "";
-    $jqueryUrl = &{ $class->tsv->{portal} } . "skins/common/js/jquery-1.10.2.js"
+    $jqueryUrl =
+      &{ $class->tsv->{portal} } . "static/bwr/jquery/dist/jquery.min.js"
       if ( $jqueryUrl eq "default" );
     $jqueryUrl = "<script type='text/javascript' src='$jqueryUrl'></script>\n"
       if ($jqueryUrl);
